@@ -14,42 +14,57 @@ def test_detect_spindle_Moelle2011():
     assert repr(detsp) == 'detsp_Moelle2011_11-18Hz_00.5-02.0s'
 
     sp = detsp(data)
-    assert len(sp.spindle) == 5
+    assert len(sp.events) == 3
+
 
-    
 def test_detect_spindle_Nir2011():
     detsp = DetectSpindle(method='Nir2011')
 
     sp = detsp(data)
-    assert len(sp.spindle) == 4    
-    
-    
+    assert len(sp.events) == 2
+
+
 def test_detect_spindle_Wamsley2012():
     detsp = DetectSpindle(method='Wamsley2012')
 
     sp = detsp(data)
-    assert len(sp.spindle) == 1    
-    
-        
+    assert len(sp.events) == 1
+
+
 def test_detect_spindle_Ferrarelli2007():
     detsp = DetectSpindle(method='Ferrarelli2007')
 
     sp = detsp(data)
-    assert len(sp.spindle) == 0
-    
+    assert len(sp.events) == 0
+
+
+def test_detect_spindle_UCSD():
+    detsp = DetectSpindle(method='UCSD')
     
+    sp = detsp(data)
+    assert len(sp.events) == 4
+
+
 def test_detect_spindle_unknownmethod():
     with raises(ValueError):
-        detsp = DetectSpindle(method='xxx')        
-        
+        detsp = DetectSpindle(method='xxx')
+
+def test_merge_close():
+    detsp = DetectSpindle(method='Nir2011')
     
+    sp = detsp(data)
+    assert len(sp.events) == 2
+
 def test_detect_spindle_to_data():
     detsp = DetectSpindle()
     sp = detsp(data)
-    
+
     sp_data = sp.to_data('count')
-    assert sp_data(0)[0] == 2
+    assert sp_data(0)[0] == 1
 
     sp_freq = sp.to_data('peak_freq')
-    assert approx(sp_freq(0)[0]) == 14.49166667
-    
\ No newline at end of file
+    assert approx(sp_freq(0)[0]) == 14.5833333
+    
+    sp_ptp = sp.to_data('ptp')
+    assert approx(sp_ptp(0)[0]) == 66.3384615
+
