@@ -5,8 +5,9 @@ from bisect import bisect_left
 from csv import writer
 from datetime import datetime, timedelta
 from itertools import compress
-from numpy import allclose, asarray, in1d, modf, isnan
+from numpy import allclose, asarray, in1d, isnan, logical_and, modf
 from math import ceil, inf
+from os.path import splitext
 from pathlib import Path
 from re import search, sub
 from scipy.io import loadmat
@@ -26,6 +27,37 @@ DOMINO_STAGE_KEY = {'N1': 'NREM1',
                     'Ar': 'Artefact',
                     'A\n': 'Artefact'}
 
+REMLOGIC_STAGE_KEY = {'S0': 'Wake',
+                      'S1': 'NREM1',
+                      'S2': 'NREM2',
+                      'S3': 'NREM3',
+                      'EM': 'REM',
+                      'ED': 'Undefined'}
+
+ALICE_STAGE_KEY = {'WK': 'Wake',
+                   'N1': 'NREM1',
+                   'N2': 'NREM2',
+                   'N3': 'NREM3',
+                   'EM': 'REM'}
+
+SANDMAN_STAGE_KEY = {' 1': 'NREM1',
+                     ' 2': 'NREM2',
+                     ' 3': 'NREM3',
+                     'em': 'REM',
+                     'ke': 'Wake',
+                     '1\t': 'NREM1',
+                     '2\t': 'NREM2',
+                     '3\t': 'NREM3',
+                     'm\t': 'REM',
+                     'e\t': 'Wake'}
+
+COMPUMEDICS_STAGE_KEY = {'?': 'Unknown',
+                         'W': 'Wake',
+                         '1': 'NREM1',
+                         '2': 'NREM2',
+                         '3': 'NREM3',
+                         'R': 'REM'}
+
 FASST_STAGE_KEY = ['Wake',
                    'NREM1',
                    'NREM2',
@@ -266,43 +298,119 @@ class Annotations():
 
         self.save()
 
-    def import_domino(self, filename, rater_name, record_start):
+    def import_staging(self, filename, source, rater_name, rec_start, 
+                       staging_start=None):
         """Import staging from a SomnoMedics Domino staging text file.
 
         Parameters
         ----------
         filename : str
-            Domino filename.
+            Staging file name.
+        source : str
+            Name of program where staging was made. One of 'domino', 'alice',
+            'compumedics', 'sandman', 'remlogic'
         rater_name : str
             Rater name for imported staging.
-        record_start : datetime
+        rec_start : datetime
             Date and time (year, month, day, hour, minute, second) of recording
-            start. Year is unimportant.
+            start. Year is ignored (New Year's Eve celebratory recordings 
+            unsupported.)
+        staging_start : datetime (default: None)
+            Date and time of staging start. For use when not provided in 
+            staging file.
         """
         if rater_name not in self.raters:
             self.add_rater(rater_name, epoch_length=30)
 
         self.get_rater(rater_name)
-
         stages = self.rater.find('stages')
 
         # list is necessary so that it does not remove in place
         for s in list(stages):
             stages.remove(s)
+        
+        if source == 'sandman':
+            encoding = 'ISO-8859-1'
+        else:
+            encoding = 'utf-8'
 
-        with open(filename, 'r') as f:
-            domino_lines = f.readlines()
-            stage_start = datetime.strptime(domino_lines[7][:8], '%H:%M:%S')
-            stage_day = int(domino_lines[1][12:14])
-            stage_month = int(domino_lines[1][15:17])
-            stage_trunc = stage_start.replace(year=1999, month=stage_month,
-                                              day=stage_day)
-            record_trunc = record_start.replace(year=1999)
-            first_second = int((record_trunc - stage_trunc).total_seconds())
-            lg.info('Time offset: ' + str(first_second))
-
+        with open(filename, 'r', encoding=encoding) as f:
+            lines = f.readlines()
             stages = self.rater.find('stages')
-            for i, line in enumerate(domino_lines[7:]):
+                        
+            if source == 'domino':            
+                stage_start = datetime.strptime(lines[7][:8], '%H:%M:%S')
+                stage_day = int(lines[1][12:14])
+                stage_month = int(lines[1][15:17])
+                stage_start_for_delta = stage_start.replace(year=1999, 
+                                                            month=stage_month,
+                                                            day=stage_day)
+                rec_start_for_delta = rec_start.replace(year=1999)
+                first_second = int((stage_start_for_delta - 
+                                    rec_start_for_delta).total_seconds())
+                
+                first_line = 7
+                
+                stage_key = DOMINO_STAGE_KEY
+                idx_stage = (14, 16)
+            
+            elif source == 'remlogic':
+                stage_start = datetime.strptime(lines[14][:19],
+                                                '%Y-%m-%dT%H:%M:%S')
+                first_second = int((stage_start - rec_start).total_seconds())
+                
+                first_line = 14
+                
+                stage_key = REMLOGIC_STAGE_KEY
+                idx_stage = (-6, -4)
+            
+            elif source == 'alice':
+                stage_start = datetime.strptime(lines[1][2:13], '%I:%M:%S %p')
+                dt = rec_start
+                
+                if lines[1][11:13] == 'pm' and rec_start.hour < 12: 
+                    # best guess in absence of date
+                    dt = rec_start - timedelta(days=1)
+                    
+                stage_start = stage_start.replace(year=dt.year,
+                                                  month=dt.month,
+                                                  day=dt.day)                                    
+                first_second = int((stage_start - rec_start).total_seconds())
+                
+                first_line = 1
+                
+                lines[-1] += '_' # to fill newline position
+                stage_key = ALICE_STAGE_KEY
+                idx_stage = (-3, -1)
+                
+            elif source == 'sandman':
+                stage_start = datetime.strptime(lines[4][12:33], 
+                                                '%d/%m/%Y %I:%M:%S %p')
+                first_second = int((stage_start - rec_start).total_seconds())
+                
+                first_line = 14
+                
+                stage_key = SANDMAN_STAGE_KEY
+                idx_stage = (-14, -12)
+                
+            elif source == 'compumedics':
+                if staging_start is None:
+                    first_second = 0
+                else:
+                    first_second = int((
+                            staging_start - rec_start).total_seconds())
+                
+                first_line = 0
+                
+                stage_key = COMPUMEDICS_STAGE_KEY
+                idx_stage = (0, 1)
+                
+            else:
+                raise ValueError('Unknown source program for staging file')
+            
+            lg.info('Time offset: ' + str(first_second) + ' sec')
+                
+            for i, one_line in enumerate(lines[first_line:]):
                 epoch = SubElement(stages, 'epoch')
 
                 start_time = SubElement(epoch, 'epoch_start')
@@ -312,13 +420,21 @@ class Annotations():
                 end_time = SubElement(epoch, 'epoch_end')
                 end_time.text = str(epoch_beg + 30)
 
-                stage = SubElement(epoch, 'stage')
-                domino_stage = DOMINO_STAGE_KEY[line[14:16]]
-                stage.text = domino_stage
+                epoch_stage = SubElement(epoch, 'stage')
+                
+                try:
+                    key = one_line[idx_stage[0]:idx_stage[1]]
+                    one_stage = stage_key[key]
+                    
+                except KeyError:
+                    one_stage = 'Unknown'
+                    lg.info('Stage not recognized: ' + key)
+                    
+                epoch_stage.text = one_stage
 
                 quality = SubElement(epoch, 'quality')
-                if domino_stage == 'Artefact':
-                    quality.text = 'Bad'
+                if one_stage == 'Artefact':
+                    quality.text = 'Poor'
                 else:
                     quality.text = 'Good'
 
@@ -572,7 +688,7 @@ class Annotations():
         stage : tuple of str, optional
             list of stages of interest
         qual : str, optional
-            epoch signal qualifier (Good or Bad)
+            epoch signal qualifier (Good or Poor)
         Returns
         -------
         list of dict
@@ -619,7 +735,9 @@ class Annotations():
 
                 if stage or qual:
                     pos = bisect_left(ep_starts, event_start)
-                    if event_start != ep_starts[pos]:
+                    if pos == len(ep_starts):
+                        pos -= 1
+                    elif event_start != ep_starts[pos]:
                         pos -= 1
 
                 if stage is None:
@@ -677,7 +795,7 @@ class Annotations():
         first_second : int, optional
             Time, in seconds from record start, at which the epochs begin
         """
-        lg.info('creating epochs')
+        lg.info('creating epochs')        
         if first_second is None:
             first_second = self.first_second
         last_sec = ceil((self.last_second - first_second) /
@@ -745,7 +863,7 @@ class Annotations():
 
         return epoch_starts[idx]
 
-    def get_stage_for_epoch(self, epoch_start, attr='stage'):
+    def get_stage_for_epoch(self, epoch_start, window_length, attr='stage'):
         """Return stage for one specific epoch.
 
         Parameters
@@ -765,6 +883,9 @@ class Annotations():
         for epoch in self.epochs:
             if epoch['start'] == epoch_start:
                 return epoch[attr]
+            elif logical_and(window_length < 30, 
+                             0 <= (epoch_start - epoch['start']) < 30):
+                return epoch[attr]
 
     def time_in_stage(self, name, attr='stage'):
         """Return time (in seconds) in the selected stage.
