@@ -8,7 +8,7 @@ from pathlib import Path
 
 from numpy import arange, asarray, empty, int64
 
-from .ioeeg import (Edf, Ktlx, BlackRock, EgiMff, FieldTrip, IEEG_org,
+from .ioeeg import (Abf, Edf, Ktlx, BlackRock, EgiMff, FieldTrip, IEEG_org,
                     Moberg, Wonambi, OpBox, Micromed, BCI2000)
 from .ioeeg.bci2000 import _read_header_length
 from .datatype import ChanTime
@@ -78,7 +78,7 @@ def detect_format(filename, server=None):
                 raise UnrecognizedFormat('Unrecognized format for directory ' +
                                          str(filename))
         else:
-            if filename.suffix == '.phy':
+            if filename.suffix == '.won':
                 return Wonambi
 
             if filename.suffix.lower() == '.trc':
@@ -87,6 +87,12 @@ def detect_format(filename, server=None):
             if filename.suffix == '.bin':  # very general
                 return OpBox
 
+            if filename.suffix == '.edf':
+                return Edf
+
+            if filename.suffix == '.abf':
+                return Abf
+
             if filename.suffix == '.dat':  # very general
                 try:
                     _read_header_length(filename)
@@ -99,16 +105,7 @@ def detect_format(filename, server=None):
 
             with filename.open('rb') as f:
                 file_header = f.read(8)
-                if file_header == b'0       ':
-                    f.seek(192)
-                    edf_type = f.read(5)
-                    if edf_type == b'EDF+C':
-                        return Edf
-                    elif edf_type == b'EDF+D':
-                        return 'EDF+D'
-                    else:
-                        return Edf
-                elif file_header in (b'NEURALCD', b'NEURALSG', b'NEURALEV'):
+                if file_header in (b'NEURALCD', b'NEURALSG', b'NEURALEV'):
                     return BlackRock
                 elif file_header[:6] == b'MATLAB':  # we might need to read more
                     return FieldTrip
@@ -164,7 +161,7 @@ class Dataset:
     Notes
     -----
     There is a difference between Dataset.filename and Dataset.dataset.filename
-    because the format is where the file that you want to read (the argument),
+    because the former is where the file that you want to read (the argument),
     while the latter is the file that you really read. There might be
     differences, for example, if the argument points to a file within a
     directory, or if the file is mapped to memory.
@@ -333,7 +330,7 @@ class Dataset:
         for i, one_begsam, one_endsam in zip(range(n_trl), begsam, endsam):
             data.axis['chan'][i] = asarray(chan, dtype='U')
             data.axis['time'][i] = (arange(one_begsam, one_endsam) /
-                                   self.header['s_freq'])
+                                    self.header['s_freq'])
 
             dataset = self.dataset
             lg.debug('begsam {0: 6}, endsam {1: 6}'.format(one_begsam,
