@@ -1,13 +1,13 @@
 """Module to detect spindles.
 """
 from logging import getLogger
-from numpy import (absolute, arange, argmax, asarray, concatenate, diff,
-                   exp, empty, floor, hstack, insert, invert, mean,
-                   median, nan, ones, pi, ptp, sqrt, square, std, vstack,
-                   where, zeros)
+from numpy import (absolute, arange, argmax, argmin, asarray, concatenate, cos,
+                   diff, exp, empty, floor, hstack, insert, invert, linspace,
+                   mean, median, nan, ones, pi, ptp, sqrt, square,
+                   std, vstack, where, zeros)
 from scipy.ndimage.filters import gaussian_filter
 from scipy.signal import (argrelmax, butter, cheby2, filtfilt, fftconvolve,
-                          hilbert, periodogram)
+                          hilbert, periodogram, tukey)
 
 from ..graphoelement import Spindles
 
@@ -28,7 +28,8 @@ class DetectSpindle:
     duration : tuple of float
         min and max duration of spindles
     """
-    def __init__(self, method='Moelle2011', frequency=None, duration=None):
+    def __init__(self, method='Moelle2011', frequency=None, duration=None,
+                 merge=True):
 
         if frequency is None:
             frequency = (11, 18)
@@ -38,17 +39,18 @@ class DetectSpindle:
         self.method = method
         self.frequency = frequency
         self.duration = duration
+        self.merge = merge
+        self.min_interval = 0
 
         self.power_peaks = 'interval'
 
         if method == 'Ferrarelli2007':
-            self.det_cheby2 = {'order': 4,
+            self.det_butter = {'order': 4,
                                'freq': self.frequency,
                                }
             self.det_wavelet = {'sd': None}
-            self.det_thresh = 2
-            self.sel_thresh = 8
-            self.min_interval = 0
+            self.det_thresh = 8
+            self.sel_thresh = 2
             self.moving_rms = {'dur': None}
             self.smooth = {'dur': None}
 
@@ -70,17 +72,35 @@ class DetectSpindle:
                                 }
             self.det_thresh = 4.5
             self.sel_thresh = None  # necessary for gui/detect
-            self.min_interval = 0
             self.moving_rms = {'dur': None}
             self.smooth = {'dur': .1}
 
+        elif method == 'UCSD':
+            self.det_wavelet = {'freqs': arange(frequency[0],
+                                                frequency[1] + .5, .5),
+                                'dur': 1,
+                                'width': .5,
+                                'win': .5,
+                                'sd': None
+                                }
+            self.det_thresh = 2  # wavelet_peak_thresh
+            self.sel_wavelet = {'freqs': arange(frequency[0],
+                                                frequency[1] + .5, .5),
+                                'dur': 1,
+                                'width': .2,
+                                'win': .2,
+                                }
+            self.sel_thresh = 1
+            self.ratio_thresh = .5
+            self.moving_rms = {'dur': None}
+            self.smooth = {'dur': None}
+
         elif method == 'Moelle2011':
             self.det_butter = {'order': 4,
                                'freq': self.frequency,
                                }
             self.det_wavelet = {'sd': None}
             self.det_thresh = 1.5
-            self.min_interval = 0
             self.sel_thresh = None
             self.moving_rms = {'dur': .2}
             self.smooth = {'dur': .2}
@@ -120,8 +140,7 @@ class DetectSpindle:
 
             if self.method == 'Ferrarelli2007':
                 sp_in_chan, values, density = detect_Ferrarelli2007(dat_orig,
-                                                                    data.s_freq,
-                                                                    time,
+                                                                    data.s_freq, time,
                                                                     self)
 
             elif self.method == 'Nir2011':
@@ -134,6 +153,10 @@ class DetectSpindle:
                                                                  data.s_freq,
                                                                  time, self)
 
+            elif self.method == 'UCSD':
+                sp_in_chan, values, density = detect_UCSD(dat_orig,
+                                                          data.s_freq, time,
+                                                          self)
             elif self.method == 'Moelle2011':
                 sp_in_chan, values, density = detect_Moelle2011(dat_orig,
                                                                 data.s_freq,
@@ -144,12 +167,17 @@ class DetectSpindle:
             spindle.det_value[i] = values['det_value']
             spindle.sel_value[i] = values['sel_value']
             spindle.density[i] = density
+
             for sp in sp_in_chan:
                 sp.update({'chan': chan})
+
             all_spindles.extend(sp_in_chan)
             # end of loop over chan
 
-        spindle.spindle = sorted(all_spindles, key=lambda x: x['start_time'])
+        spindle.events = sorted(all_spindles, key=lambda x: x['start'])
+
+        if self.merge and len(data.axis['chan'][0]) > 1:
+            spindle.events = merge_close(spindle.events, self.min_interval)
 
         return spindle
 
@@ -189,26 +217,23 @@ def detect_Ferrarelli2007(dat_orig, s_freq, time, opts):
     The original article does not specify a filter, but cheby2 seems the best
     to me.
 
-    TODO
-    ----
-    Ferrarelli, 2007, does not detect any spindles.
-
     References
     ----------
     Ferrarelli, F. et al. Am. J. Psychiatry 164, 483-92 (2007).
     """
-    dat_det = transform_signal(dat_orig, s_freq, 'cheby2', opts.det_cheby2)
+    dat_det = transform_signal(dat_orig, s_freq, 'butter', opts.det_butter)
+    dat_det = transform_signal(dat_det, s_freq, 'hilbert')
     dat_det = transform_signal(dat_det, s_freq, 'abs')
 
     det_value = define_threshold(dat_det, s_freq, 'mean', opts.det_thresh)
     sel_value = define_threshold(dat_det, s_freq, 'mean', opts.sel_thresh)
 
-    events = detect_events(dat_det, 'threshold', det_value)
+    events = detect_events(dat_det, 'above_thresh', det_value)
 
     if events is not None:
-        events = select_events(dat_det, events, 'upper_thresh', sel_value)
+        events = select_events(dat_det, events, 'above_thresh', sel_value)
 
-        events = merge_close(dat_det, events, time, opts.min_interval)
+        events = _merge_close(dat_det, events, time, opts.min_interval)
 
         events = within_duration(events, time, opts.duration)
 
@@ -222,6 +247,7 @@ def detect_Ferrarelli2007(dat_orig, s_freq, time, opts):
         sp_in_chan = []
 
     values = {'det_value': det_value, 'sel_value': sel_value}
+    lg.info(str(values))
 
     density = len(sp_in_chan) * s_freq * 30 / len(dat_orig)
 
@@ -275,10 +301,10 @@ def detect_Moelle2011(dat_orig, s_freq, time, opts):
 
     det_value = define_threshold(dat_det, s_freq, 'mean+std', opts.det_thresh)
 
-    events = detect_events(dat_det, 'threshold', det_value)
+    events = detect_events(dat_det, 'above_thresh', det_value)
 
     if events is not None:
-        events = merge_close(dat_det, events, time, opts.min_interval)
+        events = _merge_close(dat_det, events, time, opts.min_interval)
 
         events = within_duration(events, time, opts.duration)
 
@@ -354,12 +380,12 @@ def detect_Nir2011(dat_orig, s_freq, time, opts):
     det_value = define_threshold(dat_det, s_freq, 'mean+std', opts.det_thresh)
     sel_value = define_threshold(dat_det, s_freq, 'mean+std', opts.sel_thresh)
 
-    events = detect_events(dat_det, 'threshold', det_value)
+    events = detect_events(dat_det, 'above_thresh', det_value)
 
     if events is not None:
-        events = select_events(dat_det, events, 'upper_thresh', sel_value)
+        events = select_events(dat_det, events, 'above_thresh', sel_value)
 
-        events = merge_close(dat_det, events, time, opts.min_interval)
+        events = _merge_close(dat_det, events, time, opts.min_interval)
 
         events = within_duration(events, time, opts.duration)
 
@@ -417,15 +443,14 @@ def detect_Wamsley2012(dat_orig, s_freq, time, opts):
     Wamsley, E. J. et al. Biol. Psychiatry 71, 154-61 (2012).
     """
     dat_det = transform_signal(dat_orig, s_freq, 'morlet', opts.det_wavelet)
-    dat_det = transform_signal(dat_det, s_freq, 'abs')
     dat_det = transform_signal(dat_det, s_freq, 'moving_avg', opts.smooth)
 
     det_value = define_threshold(dat_det, s_freq, 'mean', opts.det_thresh)
 
-    events = detect_events(dat_det, 'threshold', det_value)
+    events = detect_events(dat_det, 'above_thresh', det_value)
 
     if events is not None:
-        events = merge_close(dat_det, events, time, opts.min_interval)
+        events = _merge_close(dat_det, events, time, opts.min_interval)
 
         events = within_duration(events, time, opts.duration)
 
@@ -445,6 +470,74 @@ def detect_Wamsley2012(dat_orig, s_freq, time, opts):
     return sp_in_chan, values, density
 
 
+def detect_UCSD(dat_orig, s_freq, time, opts):
+    """Spindle detection based on the UCSD method
+
+    Parameters
+    ----------
+    dat_orig : ndarray (dtype='float')
+        vector with the data for one channel
+    s_freq : float
+        sampling frequency
+    time : ndarray (dtype='float')
+        vector with the time points for each sample
+    opts : instance of 'DetectSpindle'
+        det_wavelet : dict
+            parameters for 'wavelet_real',
+        det_thres' : float
+            detection threshold
+        sel_thresh : float
+            selection threshold
+        duration : tuple of float
+            min and max duration of spindles
+        frequency : tuple of float
+            low and high frequency of spindle band (for power ratio)
+        ratio_thresh : float
+            ratio between power inside and outside spindle band to accept them
+
+    Returns
+    -------
+    list of dict
+        list of detected spindles
+    dict
+        'det_value' with detection value, 'sel_value' is selection value
+    float
+        spindle density, per 30-s epoch
+
+    """
+    dat_det = transform_signal(dat_orig, s_freq, 'wavelet_real',
+                               opts.det_wavelet)
+
+    det_value = define_threshold(dat_det, s_freq, 'median+std',
+                                 opts.det_thresh)
+
+    events = detect_events(dat_det, 'maxima', det_value)
+
+    dat_sel = transform_signal(dat_orig, s_freq, 'wavelet_real',
+                               opts.sel_wavelet)
+    sel_value = define_threshold(dat_sel, s_freq, 'median+std',
+                                 opts.sel_thresh)
+    events = select_events(dat_sel, events, 'above_thresh', sel_value)
+
+    events = _merge_close(dat_det, events, time, opts.min_interval)
+
+    events = within_duration(events, time, opts.duration)
+
+    events = power_ratio(events, dat_orig, s_freq, opts.frequency,
+                         opts.ratio_thresh)
+
+    power_peaks = peak_in_power(events, dat_orig, s_freq, opts.power_peaks)
+    power_avgs = avg_power(events, dat_orig, s_freq, opts.frequency)
+    sp_in_chan = make_spindles(events, power_peaks, power_avgs, dat_det,
+                               dat_orig, time, s_freq)
+
+    values = {'det_value': det_value, 'sel_value': sel_value}
+
+    density = len(sp_in_chan) * s_freq * 30 / len(dat_orig)
+
+    return sp_in_chan, values, density
+
+
 def transform_signal(dat, s_freq, method, method_opt=None):
     """Transform the data using different methods.
 
@@ -492,6 +585,16 @@ def transform_signal(dat, s_freq, method, method_opt=None):
         dur : float
             window length in number of standard deviations
 
+    morlet_real has parameters:
+        freqs : ndarray
+            vector of wavelet frequencies for spindle detection
+        dur : float
+            duration of the wavelet (sec)
+        width : float
+            wavelet width
+        win : float
+            moving average window length (sec) of wavelet convolution
+
     moving_avg has parameters:
         dur : float
             duration of the window (sec)
@@ -521,6 +624,7 @@ def transform_signal(dat, s_freq, method, method_opt=None):
         nyquist = s_freq / 2
         Wn = asarray(freq) / nyquist
         b, a = butter(N, Wn, btype='bandpass')
+        lg.info('butter: a='+str(a)+' b='+str(b)+' Wn='+str(Wn)+' N='+str(N) + ' freq: ' + str(freq))
         dat = filtfilt(b, a, dat)
 
     if 'morlet' == method:
@@ -531,6 +635,19 @@ def transform_signal(dat, s_freq, method, method_opt=None):
         wm = _wmorlet(f0, sd, s_freq, dur)
         dat = absolute(fftconvolve(dat, wm, mode='same'))
 
+    if 'wavelet_real' == method:
+        freqs = method_opt['freqs']
+        dur = method_opt['dur']
+        width = method_opt['width']
+        win = int(method_opt['win'] * s_freq)
+
+        wm = _realwavelets(s_freq, freqs, dur, width)
+        tfr = empty((dat.shape[0], wm.shape[0]))
+        for i, one_wm in enumerate(wm):
+            x = abs(fftconvolve(dat, one_wm, mode='same'))
+            tfr[:, i] = fftconvolve(x, tukey(win), mode='same')
+        dat = mean(tfr, axis=1)
+
     if 'hilbert' == method:
         dat = hilbert(dat)
 
@@ -550,9 +667,7 @@ def transform_signal(dat, s_freq, method, method_opt=None):
         rms = zeros((ldat))
 
         for i in range(ldat):
-            rms[i] = sqrt(mean(square(
-                          dat[max(0, i - halfdur):min(ldat, i + halfdur)]
-                          )))
+            rms[i] = sqrt(mean(square(dat[max(0, i - halfdur):min(ldat, i + halfdur)])))
         dat = rms
 
     if 'gaussian' == method:
@@ -598,16 +713,17 @@ def define_threshold(dat, s_freq, method, value):
 
 
 def detect_events(dat, method, value=None):
-    """Detect events using 'threshold' or 'maxima' method.
+    """Detect events using 'above_thresh', 'below_thresh' or
+    'maxima' method.
 
     Parameters
     ----------
     dat : ndarray (dtype='float')
-        vector with the data after selection-transformation
+        vector with the data after transformation
     method : str
-        'threshold' or 'maxima'
+        'above_thresh', 'below_thresh' or 'maxima'
     value : float
-        for 'threshold', it's the value of threshold for the event detection
+        for 'threshold's, it's the value of threshold for the event detection
         for 'maxima', it's the distance in s from the peak to find a minimum
 
     Returns
@@ -616,10 +732,10 @@ def detect_events(dat, method, value=None):
         N x 3 matrix with start, peak, end samples
 
     """
-    if method == 'threshold':
+    if method == 'above_thresh':
         above_det = dat >= value
         detected = _detect_start_end(above_det)
-
+        
         if detected is None:
             return None
 
@@ -628,6 +744,19 @@ def detect_events(dat, method, value=None):
         for i in detected:
             i[1] = i[0] + argmax(dat[i[0]:i[2]])
 
+    if method == 'below_thresh':
+        below_det = dat < value
+        detected = _detect_start_end(below_det)
+        
+        if detected is None:
+            return None
+
+        # add the location of the trough in the middle
+        detected = insert(detected, 1, 0, axis=1)
+        for i in detected:
+            i[1] = i[0] + argmin(dat[i[0]:i[2]])
+        #lg.info('troughs values: ' + str(dat[detected[:,1]]))
+
     if method == 'maxima':
         peaks = argrelmax(dat)[0]
         detected = vstack((peaks, peaks, peaks)).T
@@ -648,7 +777,7 @@ def select_events(dat, detected, method, value):
     detected : ndarray (dtype='int')
         N x 3 matrix with start, peak, end samples
     method : str
-        'upper_thresh', 'lower_thresh'
+        'above_thresh', 'below_thresh'
     value : float
         for 'threshold', it's the value of threshold for the spindle selection.
 
@@ -658,58 +787,66 @@ def select_events(dat, detected, method, value):
         N x 3 matrix with start, peak, end samples
 
     """
-    if method == 'upper_thresh':
+    if method == 'above_thresh':
         above_sel = dat >= value
         detected = _select_period(detected, above_sel)
-    elif method == 'lower_thresh':
+    elif method == 'below_thresh':
         below_sel = dat <= value
         detected = _select_period(detected, below_sel)
 
     return detected
 
 
-def merge_close(dat, events, time, min_interval):
-    """Merge together events separated by less than a minimum interval.
+def merge_close(events, min_interval, merge_to_longer=False):
+    """Merge events that are separated by a less than a minimum interval.
 
     Parameters
     ----------
-    dat : ndarray (dtype='float')
-        vector with the data after selection-transformation
-    events : ndarray (dtype='int')
-        N x 3 matrix with start, peak, end samples
-    time : ndarray (dtype='float')
-        vector with time points
+    events : list of dict
+        events with 'start' and 'end' times, from one or several channels.
+        **Events must be sorted by their start time.**
     min_interval : float
         minimum delay between consecutive events, in seconds
+    merge_to_longer : bool (default: False)
+        If True, info (chan, peak, etc.) from the longer of the 2 events is
+        kept. Otherwise, info from the earlier onset spindle is kept.
 
     Returns
     -------
-    ndarray (dtype='int')
-        N x 3 matrix with start, peak, end samples
+    list of dict
+        original events list with close events merged.
     """
-    if min_interval == 0:
-        return events
+    half_iv = min_interval / 2
+    merged = []
 
-    no_merge = time[events[1:, 0] - 1] - time[events[:-1, 2]] >= min_interval
+    for higher in events:
 
-    if no_merge.any():
-        begs = concatenate([[events[0, 0]], events[1:, 0][no_merge]])
-        ends = concatenate([events[:-1, 2][no_merge], [events[-1, 2]]])
+        if not merged:
+            merged.append(higher)
 
-        new_events = vstack((begs, ends)).T  # STACKED WRONG!
-    else:
-        new_events = asarray[[events[0, 0], events[0, 2]]]
+        else:
+            lower = merged[-1]
 
-    # add the location of the peak in the middle
-    new_events = insert(new_events, 1, 0, axis=1)
-    for i in new_events:
-        i[1] = i[0] + argmax(dat[i[0]:i[2]])
+            if higher['start'] - half_iv <= lower['end'] + half_iv:
 
-    return new_events
+                if merge_to_longer and (higher['end'] - higher['start'] >
+                lower['end'] - lower['start']):
+                    start = min(lower['start'], higher['start'])
+                    higher.update({'start': start})
+                    merged[-1] = higher
+                        
+                else:
+                    end = max(lower['end'], higher['end'])
+                    merged[-1].update({'end': end})
+
+            else:
+                merged.append(higher)
+
+    return merged
 
 
 def within_duration(events, time, limits):
-    """Check whether spindle is within time limits.
+    """Check whether event is within time limits.
 
     Parameters
     ----------
@@ -723,8 +860,7 @@ def within_duration(events, time, limits):
     Returns
     -------
     ndarray (dtype='int')
-        N x 3 matrix with start, peak, end samples
-
+        N x 3 matrix with start , peak, end samples
     """
     min_dur = time[events[:, 2] - 1] - time[events[:, 0]] >= limits[0]
     max_dur = time[events[:, 2] - 1] - time[events[:, 0]] <= limits[1]
@@ -732,6 +868,55 @@ def within_duration(events, time, limits):
     return events[min_dur & max_dur, :]
 
 
+def power_ratio(events, dat, s_freq, limits, ratio_thresh):
+    """Estimate the ratio in power between spindle band and lower frequencies.
+
+    Parameters
+    ----------
+    events : ndarray (dtype='int')
+        N x 3 matrix with start, peak, end samples
+    dat : ndarray (dtype='float')
+        vector with the original data
+    s_freq : float
+        sampling frequency
+    limits : tuple of float
+        high and low frequencies for spindle band
+    ratio_thresh : float
+        ratio between spindle vs non-spindle amplitude
+
+    Returns
+    -------
+    ndarray (dtype='int')
+        N x 3 matrix with start, peak, end samples
+
+    Notes
+    -----
+    In the original matlab script, it uses amplitude, not power.
+
+    """
+    ratio = empty(events.shape[0])
+    for i, one_event in enumerate(events):
+
+        x0 = one_event[0]
+        x1 = one_event[2]
+
+        if x0 < 0 or x1 >= len(dat):
+            ratio[i] = 0
+
+        else:
+            f, Pxx = periodogram(dat[x0:x1], s_freq, scaling='spectrum')
+            Pxx = sqrt(Pxx)  # use amplitude
+
+            freq_sp = (f >= limits[0]) & (f <= limits[1])
+            freq_nonsp = (f <= limits[1])
+
+            ratio[i] = mean(Pxx[freq_sp]) / mean(Pxx[freq_nonsp])
+
+    events = events[ratio > ratio_thresh, :]
+
+    return events
+
+
 def peak_in_power(events, dat, s_freq, method, value=None):
     """Define peak in power of the signal.
 
@@ -854,8 +1039,8 @@ def make_spindles(events, power_peaks, power_avgs, dat_det, dat_orig, time,
 
     spindles = []
     for i, one_peak, one_pwr in zip(events, power_peaks, power_avgs):
-        one_spindle = {'start_time': time[i[0]],
-                       'end_time': time[i[2] - 1],
+        one_spindle = {'start': time[i[0]],
+                       'end': time[i[2] - 1],
                        'peak_time': time[i[1]],
                        'peak_val': dat_det[i[1]],
                        'peak_val_orig': dat_orig[i[1]],
@@ -937,7 +1122,7 @@ def _detect_start_end(true_values):
         N x 2 matrix with starting and ending times.
     """
     neg = zeros((1), dtype='bool')
-    int_values = asarray(hstack((neg, true_values, neg)), dtype='int')
+    int_values = asarray(concatenate((neg, true_values, neg)), dtype='int')
     cross_threshold = diff(int_values)
 
     event_starts = where(cross_threshold == 1)[0]
@@ -991,6 +1176,46 @@ def _select_period(detected, true_values):
     return detected
 
 
+def _merge_close(dat, events, time, min_interval):
+    """Merge together events separated by less than a minimum interval.
+
+    Parameters
+    ----------
+    dat : ndarray (dtype='float')
+        vector with the data after selection-transformation
+    events : ndarray (dtype='int')
+        N x 3 matrix with start, peak, end samples
+    time : ndarray (dtype='float')
+        vector with time points
+    min_interval : float
+        minimum delay between consecutive events, in seconds
+
+    Returns
+    -------
+    ndarray (dtype='int')
+        N x 3 matrix with start, peak, end samples
+    """
+    if min_interval == 0:
+        return events
+
+    no_merge = time[events[1:, 0] - 1] - time[events[:-1, 2]] >= min_interval
+
+    if no_merge.any():
+        begs = concatenate([[events[0, 0]], events[1:, 0][no_merge]])
+        ends = concatenate([events[:-1, 2][no_merge], [events[-1, 2]]])
+
+        new_events = vstack((begs, ends)).T
+    else:
+        new_events = asarray[[events[0, 0], events[-1, 2]]]
+
+    # add the location of the peak in the middle
+    new_events = insert(new_events, 1, 0, axis=1)
+    for i in new_events:
+        i[1] = i[0] + argmax(dat[i[0]:i[2]])
+
+    return new_events
+
+
 def _wmorlet(f0, sd, sampling_rate, ns=5):
     """
     adapted from nitime
@@ -1010,3 +1235,34 @@ def _wmorlet(f0, sd, sampling_rate, ns=5):
     w = (exp(-t ** 2 / (2. * st ** 2)) * exp(2j * pi * f0 * t) /
          sqrt(sqrt(pi) * st * sampling_rate))
     return w
+
+
+def _realwavelets(s_freq, freqs, dur, width):
+    """Create real wavelets, for UCSD.
+
+    Parameters
+    ----------
+    s_freq : int
+        sampling frequency
+    freqs : ndarray
+        vector with frequencies of interest
+    dur : float
+        duration of the wavelets in s
+    width : float
+        parameter controlling gaussian shape
+
+    Returns
+    -------
+    ndarray
+        wavelets
+    """
+    x = arange(-dur / 2, dur / 2, 1 / s_freq)
+    wavelets = empty((len(freqs), len(x)))
+
+    g = exp(-(pi * x ** 2) / width ** 2)
+
+    for i, one_freq in enumerate(freqs):
+        y = cos(2 * pi * x * one_freq)
+        wavelets[i, :] = y * g
+
+    return wavelets
