@@ -55,13 +55,14 @@ class Info(QWidget):
     idx_end_time : QLabel
         show end time of the dataset
 
-    idx_amplitude : QLabel
-        show current amplitude
-    idx_distance : QLabel
-        show current distance between traces
+    idx_start : QLabel
+        show start time of the window
     idx_length : QLabel
         show length of the time window
-
+    idx_scaling : QLabel
+        show current scaling
+    idx_distance : QLabel
+        show current distance between traces
     """
     def __init__(self, parent):
         super().__init__()
@@ -79,9 +80,10 @@ class Info(QWidget):
         self.idx_start_time = None
         self.idx_end_time = None
         # about the visualization
-        self.idx_amplitude = None
-        self.idx_distance = None
+        self.idx_start = None
         self.idx_length = None
+        self.idx_scaling = None
+        self.idx_distance = None
 
         self.create()
         self.create_action()
@@ -111,13 +113,21 @@ class Info(QWidget):
         form = QFormLayout()
         b1.setLayout(form)
 
-        self.idx_amplitude = QLabel('')
-        self.idx_distance = QLabel('')
+        self.idx_start = QLabel('')
+        self.idx_start.setToolTip('Start time in seconds from the beginning of'
+                                  ' the recordings')
         self.idx_length = QLabel('')
+        self.idx_length.setToolTip('Duration of the time window in seconds')
+        self.idx_scaling = QLabel('')
+        self.idx_scaling.setToolTip('Global scaling for all the channels')
+        self.idx_distance = QLabel('')
+        self.idx_distance.setToolTip('Visual distances between the traces of '
+                                     'individual channels')
 
-        form.addRow('Amplitude:', self.idx_amplitude)
-        form.addRow('Distance:', self.idx_distance)
+        form.addRow('Start Time:', self.idx_start)
         form.addRow('Length:', self.idx_length)
+        form.addRow('Scaling:', self.idx_scaling)
+        form.addRow('Distance:', self.idx_distance)
 
         layout = QVBoxLayout()
         layout.addWidget(b0)
@@ -157,7 +167,7 @@ class Info(QWidget):
 
         self.action = output
 
-    def open_dataset(self, recent=None):
+    def open_dataset(self, recent=None, debug_filename=None):
         """Open a new dataset.
 
         Parameters
@@ -186,6 +196,10 @@ class Info(QWidget):
                 filename = recent
                 repo = None
 
+        elif debug_filename is not None:
+            filename = debug_filename
+            repo = None
+
         else:
             repo = None
 
@@ -216,8 +230,8 @@ class Info(QWidget):
             elif file_or_dir == 'abort':
                 return
 
-            if filename == '':
-                return
+        if filename == '':
+            return
 
         # clear previous dataset once the user opens another dataset
         if self.dataset is not None:
@@ -225,16 +239,13 @@ class Info(QWidget):
 
         self.parent.statusBar().showMessage('Reading dataset: ' +
                                             basename(filename))
-        lg.info(str(filename))
         try:
             self.filename = filename
             self.repo = repo
             if repo == 'ieeg.org':
                 ieeg_org.SESS = ieeg_org.Session(username=username,
                                                  password_md5=password)
-
             self.dataset = Dataset(filename, server=repo)
-
         except FileNotFoundError:
             self.parent.statusBar().showMessage('File ' + basename(filename) +
                                                 ' cannot be read')
@@ -259,15 +270,16 @@ class Info(QWidget):
         self.idx_n_chan.setText(str(len(header['chan_name'])))
         start_time = header['start_time'].strftime('%b-%d %H:%M:%S')
         self.idx_start_time.setText(start_time)
-        end_time = header['start_time'] + timedelta(seconds=header['n_samples']
-                                                    / header['s_freq'])
+        end_time = (header['start_time'] +
+                    timedelta(seconds=header['n_samples'] / header['s_freq']))
         self.idx_end_time.setText(end_time.strftime('%b-%d %H:%M:%S'))
 
     def display_view(self):
         """Update information about the size of the traces."""
-        self.idx_amplitude.setText(str(self.parent.value('y_scale')))
-        self.idx_distance.setText(str(self.parent.value('y_distance')))
+        self.idx_start.setText(str(self.parent.value('window_start')))
         self.idx_length.setText(str(self.parent.value('window_length')))
+        self.idx_scaling.setText(str(self.parent.value('y_scale')))
+        self.idx_distance.setText(str(self.parent.value('y_distance')))
 
     def reset(self):
         """Reset widget to original state."""
@@ -282,9 +294,10 @@ class Info(QWidget):
         self.idx_end_time.setText('')
 
         # about the visualization
-        self.idx_amplitude.setText('')
+        self.idx_scaling.setText('')
         self.idx_distance.setText('')
         self.idx_length.setText('')
+        self.idx_start.setText('')
 
 
 class Remote(QDialog):
