@@ -1,16 +1,19 @@
 """Definition of the main widgets, with recordings.
 """
-from datetime import timedelta
+from datetime import time, datetime, timedelta
 from functools import partial
 from logging import getLogger
+from re import compile
 
-from numpy import (abs, arange, argmin, asarray, ceil, empty, floor, in1d,
-                   max, min, linspace, log2, nanmean, pad, power)
+from numpy import (abs, arange, argmin, around, asarray, ceil, empty, floor,
+                   in1d, max, min, linspace, log2, logical_or, nanmean, pad,
+                   power)
 
 from PyQt5.QtCore import QPointF, Qt, QRectF
 from PyQt5.QtGui import (QBrush,
                          QColor,
                          QIcon,
+                         QKeyEvent,
                          QKeySequence,
                          QPen,
                          )
@@ -22,6 +25,7 @@ from PyQt5.QtWidgets import (QAction,
                              QGraphicsSimpleTextItem,
                              QGraphicsView,
                              QGroupBox,
+                             QInputDialog,
                              QVBoxLayout,
                              )
 
@@ -33,6 +37,7 @@ from .utils import (convert_name_to_color,
                     LINE_COLOR,
                     LINE_WIDTH,
                     Path,
+                    RectMarker,
                     TextItem_with_BG,
                     )
 
@@ -47,6 +52,8 @@ NoPen.setStyle(Qt.NoPen)
 
 MINIMUM_N_SAMPLES = 32  # at least this number of samples to compute fft
 
+CHECK_TIME_STR = compile('[0-9:-]+$')
+
 
 class ConfigTraces(Config):
     """Widget with preferences in Settings window for Overview."""
@@ -81,12 +88,14 @@ class ConfigTraces(Config):
         self.index['grid_x'] = FormBool('Grid on time axis')
         self.index['grid_xtick'] = FormFloat()
         self.index['grid_y'] = FormBool('Grid on voltage axis')
+        self.index['grid_ytick'] = FormFloat()
 
         form_layout = QFormLayout()
         box1.setLayout(form_layout)
         form_layout.addRow(self.index['grid_x'])
         form_layout.addRow('Tick every (s)', self.index['grid_xtick'])
         form_layout.addRow(self.index['grid_y'])
+        form_layout.addRow('Lines at + and - (uV):', self.index['grid_ytick'])
 
         box2 = QGroupBox('Current Window')
         self.index['window_start'] = FormInt()
@@ -175,6 +184,12 @@ class Traces(QGraphicsView):
         self.idx_info = None
         self.idx_markers = []
         self.idx_annot = []
+        self.idx_annot_labels = []
+        self.cross_chan_mrk = True
+        self.highlight = None
+        self.event_sel = None
+        self.deselect = None
+        self.ready = True
 
         self.create_action()
 
@@ -183,25 +198,34 @@ class Traces(QGraphicsView):
         actions = {}
 
         act = QAction(QIcon(ICON['step_prev']), 'Previous Step', self)
-        act.setShortcut(QKeySequence.MoveToPreviousChar)
+        act.setShortcut('[')
         act.triggered.connect(self.step_prev)
         actions['step_prev'] = act
 
         act = QAction(QIcon(ICON['step_next']), 'Next Step', self)
-        act.setShortcut(QKeySequence.MoveToNextChar)
+        act.setShortcut(']')
         act.triggered.connect(self.step_next)
         actions['step_next'] = act
 
         act = QAction(QIcon(ICON['page_prev']), 'Previous Page', self)
-        act.setShortcut(QKeySequence.MoveToPreviousPage)
+        act.setShortcut(QKeySequence.MoveToPreviousChar)
         act.triggered.connect(self.page_prev)
         actions['page_prev'] = act
 
         act = QAction(QIcon(ICON['page_next']), 'Next Page', self)
-        act.setShortcut(QKeySequence.MoveToNextPage)
+        act.setShortcut(QKeySequence.MoveToNextChar)
         act.triggered.connect(self.page_next)
         actions['page_next'] = act
 
+        act = QAction('Go to Epoch', self)
+        act.setShortcut(QKeySequence.FindNext)
+        act.triggered.connect(self.go_to_epoch)
+        actions['go_to_epoch'] = act
+        
+        act = QAction('Line Up with Epoch', self)
+        act.triggered.connect(self.line_up_with_epoch)
+        actions['line_up_with_epoch'] = act
+
         act = QAction(QIcon(ICON['zoomprev']), 'Wider Time Window', self)
         act.setShortcut(QKeySequence.ZoomIn)
         act.triggered.connect(self.X_more)
@@ -212,12 +236,12 @@ class Traces(QGraphicsView):
         act.triggered.connect(self.X_less)
         actions['X_less'] = act
 
-        act = QAction(QIcon(ICON['zoomin']), 'Larger Amplitude', self)
+        act = QAction(QIcon(ICON['zoomin']), 'Larger Scaling', self)
         act.setShortcut(QKeySequence.MoveToPreviousLine)
         act.triggered.connect(self.Y_more)
         actions['Y_less'] = act
 
-        act = QAction(QIcon(ICON['zoomout']), 'Smaller Amplitude', self)
+        act = QAction(QIcon(ICON['zoomout']), 'Smaller Scaling', self)
         act.setShortcut(QKeySequence.MoveToNextLine)
         act.triggered.connect(self.Y_less)
         actions['Y_more'] = act
@@ -254,6 +278,12 @@ class Traces(QGraphicsView):
         act.triggered.connect(partial(self.add_time, 6 * 60 * 60))
         actions['addtime_6h'] = act
 
+        act = QAction('Full-length markers', self)
+        act.setCheckable(True)
+        act.setChecked(True)
+        act.triggered.connect(self.display_annotations)
+        actions['cross_chan_mrk'] = act
+
         self.action = actions
 
     def read_data(self):
@@ -312,6 +342,7 @@ class Traces(QGraphicsView):
 
         self.idx_markers = []
         self.idx_annot = []
+        self.idx_annot_labels = []
 
         self.add_chan_labels()
         self.add_time_labels()
@@ -434,11 +465,28 @@ class Traces(QGraphicsView):
                                  Qt.DotLine))
 
         if self.parent.value('grid_y'):
+            y_tick = (self.parent.value('grid_ytick') *
+                      self.parent.value('y_scale'))
             for one_label_item in self.idx_label:
                 x_pos = [window_start, window_end]
-                y_pos = [one_label_item.y(), one_label_item.y()]
-                path = self.scene.addPath(Path(x_pos, y_pos))
-                path.setPen(QPen(QColor(LINE_COLOR), LINE_WIDTH, Qt.DotLine))
+                y = one_label_item.y()
+
+                y_pos_0 = [y, y]
+                path_0 = self.scene.addPath(Path(x_pos, y_pos_0))
+                path_0.setPen(QPen(QColor(LINE_COLOR), LINE_WIDTH,
+                                   Qt.DotLine))
+
+                y_up = one_label_item.y() + y_tick
+                y_pos_up = [y_up, y_up]
+                path_up = self.scene.addPath(Path(x_pos, y_pos_up))
+                path_up.setPen(QPen(QColor(LINE_COLOR), LINE_WIDTH,
+                                    Qt.DotLine))
+
+                y_down = one_label_item.y() - y_tick
+                y_pos_down = [y_down, y_down]
+                path_down = self.scene.addPath(Path(x_pos, y_pos_down))
+                path_down.setPen(QPen(QColor(LINE_COLOR), LINE_WIDTH,
+                                      Qt.DotLine))
 
     def display_markers(self):
         """Add markers on top of first plot."""
@@ -486,12 +534,15 @@ class Traces(QGraphicsView):
         for item in self.idx_annot:
             self.scene.removeItem(item)
         self.idx_annot = []
+        for item in self.idx_annot_labels:
+            self.scene.removeItem(item)
+        self.idx_annot_labels = []
+        self.highlight = None
 
         window_start = self.parent.value('window_start')
         window_length = self.parent.value('window_length')
         window_end = window_start + window_length
         y_distance = self.parent.value('y_distance')
-        raw_chan_name = list(map(take_raw_name, self.chan))
 
         bookmarks = []
         events = []
@@ -514,9 +565,9 @@ class Traces(QGraphicsView):
                 if annot in events:
                     color = convert_name_to_color(annot['name'])
 
-                if annot['chan'] == ['']:
+                if logical_or(annot['chan'] == [''],
+                              self.action['cross_chan_mrk'].isChecked()):
                     h_annot = len(self.idx_label) * y_distance
-                    y_annot = (0, )
 
                     item = TextItem_with_BG(color.darker(200))
                     item.setText(annot['name'])
@@ -525,44 +576,69 @@ class Traces(QGraphicsView):
                     item.setFlag(QGraphicsItem.ItemIgnoresTransformations)
                     item.setRotation(-90)
                     self.scene.addItem(item)
-                    self.idx_annot.append(item)
-                    zvalue = -8
+                    self.idx_annot_labels.append(item)
 
-                else:
-                    h_annot = y_distance
-                    # find indices of channels with annotations
-                    chan_idx_in_mrk = in1d(raw_chan_name, annot['chan'])
-                    y_annot = asarray(self.chan_pos)[chan_idx_in_mrk]
-                    y_annot -= y_distance / 2
-                    zvalue = -7
+                    item = RectMarker(mrk_start, 0, mrk_end - mrk_start, 
+                                      h_annot, zvalue=-8, 
+                                      color=color.lighter(120))
 
-                for y in y_annot:
-                    item = QGraphicsRectItem(mrk_start, y,
-                                             mrk_end - mrk_start, h_annot)
-                    item.setPen(color)
-                    item.setBrush(color)
-                    item.setZValue(zvalue)
+                    """
+                    item = QGraphicsRectItem(mrk_start + 0.5, 0,
+                                             mrk_end - mrk_start - 1, 
+                                             h_annot)
+                    item.setBrush(color.lighter(120))
+                    item.setPen(color.lighter(120))
+                    item.setZValue(-8)
+                    """
                     self.scene.addItem(item)
                     self.idx_annot.append(item)
 
+                if annot['chan'] != ['']:
+                    # find indices of channels with annotations
+                    chan_idx_in_mrk = in1d(self.chan, annot['chan'])
+                    y_annot = asarray(self.chan_pos)[chan_idx_in_mrk]
+                    y_annot -= y_distance / 2
+
+                    for y in y_annot:
+                        item = RectMarker(mrk_start, y, mrk_end - mrk_start, 
+                                          y_distance, zvalue=-7, color=color)
+                        self.scene.addItem(item)
+                        self.idx_annot.append(item)
+                        
+                        """
+                        item = QGraphicsRectItem(mrk_start + 0.5, y,
+                                                 mrk_end - mrk_start - 1,
+                                                 y_distance)
+                        item.setBrush(color)
+                        item.setPen(color)
+                        item.setZValue(-7)
+                        self.scene.addItem(item)
+                        self.idx_annot.append(item)
+                        """
+
     def step_prev(self):
         """Go to the previous step."""
-        window_start = (self.parent.value('window_start') -
-                        self.parent.value('window_length') /
-                        self.parent.value('window_step'))
+        window_start = around(self.parent.value('window_start') -
+                              self.parent.value('window_length') /
+                              self.parent.value('window_step'), 2)
+        if window_start < 0:
+            return
         self.parent.overview.update_position(window_start)
 
     def step_next(self):
         """Go to the next step."""
-        window_start = (self.parent.value('window_start') +
-                        self.parent.value('window_length') /
-                        self.parent.value('window_step'))
+        window_start = around(self.parent.value('window_start') +
+                              self.parent.value('window_length') /
+                              self.parent.value('window_step'), 2)
+
         self.parent.overview.update_position(window_start)
 
     def page_prev(self):
         """Go to the previous page."""
         window_start = (self.parent.value('window_start') -
                         self.parent.value('window_length'))
+        if window_start < 0:
+            return
         self.parent.overview.update_position(window_start)
 
     def page_next(self):
@@ -571,6 +647,41 @@ class Traces(QGraphicsView):
                         self.parent.value('window_length'))
         self.parent.overview.update_position(window_start)
 
+    def go_to_epoch(self, checked=False, test_text_str=None):
+        """Go to any window"""
+        if test_text_str is not None:
+            time_str = test_text_str
+            ok = True
+        else:
+            time_str, ok = QInputDialog.getText(self,
+                                                'Go To Epoch',
+                                                'Enter start time of the '
+                                                'epoch,\nin seconds ("1560") '
+                                                'or\nas absolute time '
+                                                '("22:30")')
+
+        if not ok:
+            return
+
+        try:
+            rec_start_time = self.parent.info.dataset.header['start_time']
+            window_start = _convert_timestr_to_seconds(time_str, rec_start_time)
+        except ValueError as err:
+            self.parent.statusBar().showMessage(str(err))
+            return
+        self.parent.overview.update_position(window_start)
+        
+    def line_up_with_epoch(self):
+        """Go to the start of the present epoch."""
+        if self.parent.notes.annot is None:  # TODO: remove if buttons are disabled
+            self.parent.statusBar().showMessage('No score file loaded')
+            return
+        
+        new_window_start = self.parent.notes.annot.get_epoch_start(
+                self.parent.value('window_start'))
+        
+        self.parent.overview.update_position(new_window_start)
+
     def add_time(self, extra_time):
         """Go to the predefined time forward."""
         window_start = self.parent.value('window_start') + extra_time
@@ -578,6 +689,8 @@ class Traces(QGraphicsView):
 
     def X_more(self):
         """Zoom in on the x-axis."""
+        if self.parent.value('window_length') < 0.3:
+            return
         self.parent.value('window_length',
                           self.parent.value('window_length') * 2)
         self.parent.overview.update_position()
@@ -594,17 +707,17 @@ class Traces(QGraphicsView):
         self.parent.overview.update_position()
 
     def Y_more(self):
-        """Increase the amplitude."""
+        """Increase the scaling."""
         self.parent.value('y_scale', self.parent.value('y_scale') * 2)
         self.parent.traces.display()
 
     def Y_less(self):
-        """Decrease the amplitude."""
+        """Decrease the scaling."""
         self.parent.value('y_scale', self.parent.value('y_scale') / 2)
         self.parent.traces.display()
 
     def Y_ampl(self, new_y_scale):
-        """Make amplitude on Y axis using predefined values"""
+        """Make scaling on Y axis using predefined values"""
         self.parent.value('y_scale', new_y_scale)
         self.parent.traces.display()
 
@@ -634,6 +747,18 @@ class Traces(QGraphicsView):
         if not self.scene:
             return
 
+        if self.event_sel:
+            self.deselect = True
+            self.event_sel = None
+            highlight = self.highlight
+            self.scene.removeItem(highlight)
+            self.highlight = None
+            self.parent.statusBar().showMessage('')
+            return
+
+        self.ready = False
+        self.event_sel = None
+
         xy_scene = self.mapToScene(event.pos())
         chan_idx = argmin(abs(asarray(self.chan_pos) - xy_scene.y()))
         self.sel_chan = chan_idx
@@ -646,6 +771,50 @@ class Traces(QGraphicsView):
             channame = self.chan[self.sel_chan] + ' in selected window'
             self.parent.spectrum.show_channame(channame)
 
+        # JOB: Make annotations clickable
+        else:
+            for annot in self.idx_annot:
+                if annot.contains(xy_scene):
+                    beg = annot.rect().x() - 0.5
+                    end = beg + annot.rect().width() + 1
+                    msg = 'Event from ' + str(beg) + ' to ' + str(end)
+                    self.parent.statusBar().showMessage(msg)
+                    self.event_sel = annot
+                    highlight = QGraphicsRectItem(annot.rect().x() - 0.5,
+                                                  annot.rect().y() - 0.5,
+                                                  annot.rect().width() + 1,
+                                                  annot.rect().height() + 1)
+                    pen = QPen(QColor(255, 255, 51), 0, Qt.SolidLine,
+                               Qt.RoundCap)
+                    brush = QBrush(QColor(255, 255, 51))
+                    highlight.setPen(pen)
+                    highlight.setBrush(brush)
+                    highlight.setZValue(-5)
+                    self.scene.addItem(highlight)
+                    self.highlight = highlight
+                    break
+
+        # self.display_annotations
+        self.ready = True
+
+        """
+    def mouseDoubleClickEvent(self, event):
+        When an event marker is double-clicked, it is removed
+        if not self.scene:
+            return
+
+        chk_marker = self.parent.notes.action['new_bookmark'].isChecked()
+        chk_event = self.parent.notes.action['new_event'].isChecked()
+
+        if not (chk_marker or chk_event):
+            xy_scene = self.mapToScene(event.pos())
+            for annot in self.idx_annot:
+                if annot.contains(xy_scene):
+                    self.parent.statusBar().showMessage('You clicked on an event!')
+                    self.parent.notes.remove_event(self, time=annot.mapToScene(0,0))
+                    self.scene.removeItem(annot)
+        """
+
     def mouseMoveEvent(self, event):
         """When normal selection, update power spectrum with current selection.
         Otherwise, show the range of the new marker.
@@ -653,6 +822,9 @@ class Traces(QGraphicsView):
         if not self.scene:
             return
 
+        if self.event_sel or self.deselect:
+            return
+
         if self.idx_sel in self.scene.items():
             self.scene.removeItem(self.idx_sel)
             self.idx_sel = None
@@ -728,8 +900,19 @@ class Traces(QGraphicsView):
         if not self.scene:
             return
 
+        if self.event_sel:
+            return
+
+        if self.deselect:
+            self.deselect = False
+            return
+
+        if not self.ready:
+            return
+
         chk_marker = self.parent.notes.action['new_bookmark'].isChecked()
         chk_event = self.parent.notes.action['new_event'].isChecked()
+        y_distance = self.parent.value('y_distance')
 
         if chk_marker or chk_event:
 
@@ -755,9 +938,11 @@ class Traces(QGraphicsView):
                 if chk_marker:
                     self.parent.notes.add_bookmark(time)
 
-                elif chk_event:
+                elif chk_event and start != end:
                     eventtype = self.parent.notes.idx_eventtype.currentText()
-                    self.parent.notes.add_event(eventtype, time)
+                    chan_idx = int(floor(self.sel_xy[1] / y_distance))
+                    chan = self.chan[chan_idx]
+                    self.parent.notes.add_event(eventtype, time, chan)
 
         else:  # normal selection
 
@@ -777,6 +962,25 @@ class Traces(QGraphicsView):
             self.scene.removeItem(self.idx_sel)
             self.idx_sel = None
 
+    def keyPressEvent(self, event):
+        if not self.event_sel:
+            return
+
+        annot = self.event_sel
+        highlight = self.highlight
+        annot_start = annot.rect().x() - 0.5
+        annot_end = annot_start + annot.rect().width() + 1
+
+        if type(event) == QKeyEvent and (
+           event.key() == Qt.Key_Delete or event.key() == Qt.Key_Backspace):
+            self.parent.notes.remove_event(time=(annot_start, annot_end))
+            self.scene.removeItem(highlight)
+            msg = 'Deleted event from ' + str(annot_start) + ' to ' + str(annot_end)
+            self.parent.statusBar().showMessage(msg)
+            self.event_sel = None
+            self.highlight = None
+            self.display_annotations
+
     def resizeEvent(self, event):
         """Resize scene so that it fits the whole widget.
 
@@ -896,9 +1100,9 @@ def _select_channels(data, channels):
 
     Notes
     -----
-    This function does the same as wonambi.trans.select, but it's much faster.
-    wonambi.trans.Select needs to flexible for any data type, here we assume
-    that we have one trial, and that channel is the first dimension.
+    This function does the same as wonambi.trans.select, but it's much
+    faster. wonambi.trans.Select needs to flexible for any data type, here
+    we assume that we have one trial, and that channel is the first dimension.
 
     """
     output = data._copy()
@@ -908,3 +1112,47 @@ def _select_channels(data, channels):
     output.axis['chan'][0] = asarray(channels)
 
     return output
+
+
+def _convert_timestr_to_seconds(time_str, rec_start):
+    """Convert input from user about time string to an absolute time for
+    the recordings.
+
+    Parameters
+    ----------
+    time_str : str
+        time information as '123' or '22:30' or '22:30:22'
+    rec_start: instance of datetime
+        absolute start time of the recordings.
+
+    Returns
+    -------
+    int
+        start time of the window, in s, from the start of the recordings
+
+    Raises
+    ------
+    ValueError
+        if it cannot convert the string
+    """
+    if not CHECK_TIME_STR.match(time_str):
+        raise ValueError('Input can only contain digits and colons')
+
+    if ':' in time_str:
+        time_split = [int(x) for x in time_str.split(':')]
+
+        # if it's in 'HH:MM' format, add ':SS'
+        if len(time_split) == 2:
+            time_split.append(0)
+        clock_time = time(*time_split)
+
+        chosen_start = datetime.combine(rec_start.date(), clock_time)
+        # if the clock time is after start of the recordings, assume it's the next day
+        if clock_time < rec_start.time():
+            chosen_start += timedelta(days=1)
+
+        window_start = int((chosen_start - rec_start).total_seconds())
+    else:
+        window_start = int(time_str)
+
+    return window_start
